@@ -21,19 +21,9 @@
 
 #include <config.h>
 
-/* system headers */
 #include <sys/file.h>
-#include <sys/stat.h>
-/* X11 headers */
 #include <X11/X.h>
 
-#ifdef HAVE_XFREE
-#include <X11/XF86keysym.h>
-#endif
-#ifdef HAVE_XSUN
-#include <X11/Sunkeysym.h>
-#endif
-
 /* Gnome headers */
 #include <gdk/gdkx.h>
 #include <gnome.h>
@@ -156,7 +146,7 @@ execute (char *cmd, gboolean sync)
 {
 	gboolean retval;
 
-	if (sync == TRUE)
+	if (sync != FALSE)
 		retval = g_spawn_command_line_sync
 			(cmd, NULL, NULL, NULL, NULL);
 	else
@@ -502,7 +492,7 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 				break;
 			}
 
-			if (key_code > 0 && grab == TRUE)
+			if (key_code > 0 && grab != FALSE)
 			{
 				grab_key (acme, key_code);
 				keys[i].key_code = key_code;
@@ -511,11 +501,10 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 	}
 }
 
-static gboolean
+static void
 init_kbd (Acme *acme)
 {
 	int i;
-	gboolean retval = FALSE;
 
 	acme->display = gdk_display_get_default ();
 	acme->screens = NULL;
@@ -575,7 +564,7 @@ init_kbd (Acme *acme)
 			break;
 		}
 
-		if (tmp > 0 && grab == TRUE)
+		if (tmp > 0 && grab != FALSE)
 		{
 #ifdef DEBUG
 			g_print ("grabbed key %d for gconf key %s\n",
@@ -583,7 +572,6 @@ init_kbd (Acme *acme)
 					keys[i].key_config);
 #endif
 			grab_key (acme, keys [i].key_code);
-			retval = TRUE;
 		}
 	}
 
@@ -595,7 +583,7 @@ init_kbd (Acme *acme)
 				acme, NULL, NULL);
 	}
 
-	return retval;
+	return;
 }
 
 static void
@@ -796,7 +784,7 @@ do_close_window_action (Acme *acme)
 	for (item = windows; item != NULL; item = item->next)
 	{
 		WnckWindow *window = (WnckWindow *) (item->data);
-		if (wnck_window_is_active (window) == TRUE)
+		if (wnck_window_is_active (window) != FALSE)
 		{
 			focused = window;
 			break;
@@ -822,7 +810,7 @@ do_shade_window_action (Acme *acme)
 	for (item = windows; item != NULL; item = item->next)
 	{
 		WnckWindow *window = (WnckWindow *) (item->data);
-		if (wnck_window_is_active (window) == TRUE)
+		if (wnck_window_is_active (window) != FALSE)
 		{
 			focused = window;
 			break;
@@ -831,24 +819,92 @@ do_shade_window_action (Acme *acme)
 
 	if (focused != NULL)
 	{
-		if (wnck_window_is_shaded (focused) == TRUE)
+		if (wnck_window_is_shaded (focused) != FALSE)
 			wnck_window_unshade (focused);
 		else
 			wnck_window_shade (focused);
 	}
 }
 
+static void
+do_unknown_action (Acme *acme, const char *url)
+{
+	char *string, *command;
+
+	g_return_if_fail (url != NULL);
+
+	string = gconf_client_get_string (acme->conf_client,
+			"/desktop/gnome/url-handlers/unknown/command",
+			NULL);
+
+	if (string == NULL || strcmp (string, "") == 0)
+		return;
+
+	command = g_strdup_printf (string, url);
+
+	execute (command, FALSE);
+
+	g_free (command);
+	g_free (string);
+}
+
+static void
+do_help_action (Acme *acme)
+{
+	char *string, *command;
+
+	string = gconf_client_get_string (acme->conf_client,
+			"/desktop/gnome/url-handlers/ghelp/command",
+			NULL);
+
+	if (string == NULL && strcmp (string, "") == 0)
+	{
+		do_unknown_action (acme, "ghelp:");
+		return;
+	}
+
+	command = g_strdup_printf (string, "");
+
+	execute (command, FALSE);
+
+	g_free (command);
+	g_free (string);
+}
+
+static void
+do_mail_action (Acme *acme)
+{
+	char *string, *command;
+
+	string = gconf_client_get_string (acme->conf_client,
+			"/desktop/gnome/url-handlers/mailto/command",
+			NULL);
+
+	if (string == NULL || strcmp (string, "") == 0)
+		return;
+
+	command = g_strdup_printf (string, "");
+
+	execute (command, FALSE);
+
+	g_free (command);
+	g_free (string);
+}
+
 static void
 do_www_action (Acme *acme, const char *url)
 {
 	char *string, *command;
 
 	string = gconf_client_get_string (acme->conf_client,
-		"/desktop/gnome/url-handlers/unknown/command",
+		"/desktop/gnome/url-handlers/http/command",
 		 NULL);
 
-	if (string == NULL && strcmp (string, "") == 0)
+	if (string == NULL || strcmp (string, "") == 0)
+	{
+		do_unknown_action (acme, url ? url : "");
 		return;
+	}
 
 	if (url == NULL)
 		command = g_strdup_printf (string, "about:blank");
@@ -913,7 +969,7 @@ do_media_action (Acme *acme)
 	GnomeVFSMimeApplication *app;
 
 	app = gnome_vfs_mime_get_default_application ("audio/x-mp3");
-	if (app->requires_terminal == TRUE || app->command == NULL)
+	if (app->requires_terminal != FALSE || app->command == NULL)
 		return;
 
 	execute (app->command, FALSE);
@@ -1043,7 +1099,7 @@ do_action (int type, Acme *acme)
 		execute ("gnome-search-tool", FALSE);
 		break;
 	case EMAIL_KEY:
-		execute_this_or_that ("evolution mailto:", "balsa");
+		do_mail_action (acme);
 		break;
 	case SLEEP_KEY:
 		execute_this_or_that ("apm", "xset dpms force off");
@@ -1055,7 +1111,7 @@ do_action (int type, Acme *acme)
 		execute ("gnucash", FALSE);
 		break;
 	case HELP_KEY:
-		execute ("yelp", FALSE);
+		do_help_action (acme);
 		break;
 	case WWW_KEY:
 		do_www_action (acme, NULL);
@@ -1098,9 +1154,6 @@ acme_filter_events (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 	GList *l;
 	int i;
 
-	if (xev->type != KeyRelease)
-		return GDK_FILTER_CONTINUE;
-
 	/* Look for which screen we're receiving events */
 	for (l = acme->screens; (l != NULL) && (event_screen == NULL);
 			l = l->next)
@@ -1137,6 +1190,19 @@ acme_filter_events (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 			case NEXT_KEY:
 			case REFRESH_KEY:
 				return GDK_FILTER_CONTINUE;
+			case VOLUME_DOWN_KEY:
+			case VOLUME_UP_KEY:
+#ifdef USE_FBLEVEL
+			case BRIGHT_DOWN_KEY:
+			case BRIGHT_UP_KEY:
+#endif
+				/* auto-repeatable keys */
+				if (xev->type != KeyPress)
+					return GDK_FILTER_CONTINUE;
+				break;
+			default:
+				if (xev->type != KeyRelease)
+					return GDK_FILTER_CONTINUE;
 			}
 
 			acme->current_screen = event_screen;
@@ -1192,11 +1258,7 @@ main (int argc, char *argv[])
 			GCONF_CLIENT_PRELOAD_ONELEVEL,
 			NULL);
 
-	if (init_kbd (acme) == FALSE)
-	{
-		execute ("acme-properties", FALSE);
-		exit (0);
-	}
+	init_kbd (acme);
 	init_sm (acme);
 	init_tray (acme);
 	acme->current_screen = gdk_screen_get_default ();
